from enum import Enum


class FileOutputFormat(str, Enum):
    STL = 'stl'
    OBJ = 'obj'
    DAE = 'dae'
    STEP = 'step'
    FBX = 'fbx'
    FBXB = 'fbxb'

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ApiCallStatus(str, Enum):
    QUEUED = 'Queued'
    UPLOADED = 'Uploaded'
    IN_PROGRESS = 'In Progress'
    COMPLETED = 'Completed'
    FAILED = 'Failed'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.o_auth2_grant_type import OAuth2GrantType
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeviceAccessTokenRequestForm")


@attr.s(auto_attribs=True)
class DeviceAccessTokenRequestForm:
    """ """
    client_id: Union[Unset, str] = UNSET
    device_code: Union[Unset, str] = UNSET
    grant_type: Union[Unset, OAuth2GrantType] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        client_id = self.client_id
        device_code = self.device_code
        grant_type: Union[Unset, str] = UNSET
        if not isinstance(self.grant_type, Unset):
            grant_type = self.grant_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if client_id is not UNSET:
            field_dict['client_id'] = client_id
        if device_code is not UNSET:
            field_dict['device_code'] = device_code
        if grant_type is not UNSET:
            field_dict['grant_type'] = grant_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        client_id = d.pop("client_id", UNSET)

        device_code = d.pop("device_code", UNSET)

        _grant_type = d.pop("grant_type", UNSET)
        grant_type: Union[Unset, OAuth2GrantType]
        if isinstance(_grant_type, Unset):
            grant_type = UNSET
        else:
            grant_type = OAuth2GrantType(_grant_type)

        device_access_token_request_form = cls(
            client_id=client_id,
            device_code=device_code,
            grant_type=grant_type,
        )

        device_access_token_request_form.additional_properties = d
        return device_access_token_request_form

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OAuth2GrantType(str, Enum):
    URN_IETF_PARAMS_OAUTH_GRANT_TYPE_DEVICE_CODE = 'urn:ietf:params:oauth:grant-type:device_code'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.cache_metadata import CacheMetadata
from ..models.environment import Environment
from ..models.file_system_metadata import FileSystemMetadata
from ..models.connection import Connection
from ..types import UNSET, Unset

T = TypeVar("T", bound="EngineMetadata")


@attr.s(auto_attribs=True)
class EngineMetadata:
    """ """
    async_jobs_running: Union[Unset, bool] = False
    cache: Union[Unset, CacheMetadata] = UNSET
    environment: Union[Unset, Environment] = UNSET
    fs: Union[Unset, FileSystemMetadata] = UNSET
    git_hash: Union[Unset, str] = UNSET
    pubsub: Union[Unset, Connection] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        async_jobs_running = self.async_jobs_running
        cache: Union[Unset, str] = UNSET
        if not isinstance(self.cache, Unset):
            cache = self.cache.value
        environment: Union[Unset, str] = UNSET
        if not isinstance(self.environment, Unset):
            environment = self.environment.value
        fs: Union[Unset, str] = UNSET
        if not isinstance(self.fs, Unset):
            fs = self.fs.value
        git_hash = self.git_hash
        pubsub: Union[Unset, str] = UNSET
        if not isinstance(self.pubsub, Unset):
            pubsub = self.pubsub.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if async_jobs_running is not UNSET:
            field_dict['async_jobs_running'] = async_jobs_running
        if cache is not UNSET:
            field_dict['cache'] = cache
        if environment is not UNSET:
            field_dict['environment'] = environment
        if fs is not UNSET:
            field_dict['fs'] = fs
        if git_hash is not UNSET:
            field_dict['git_hash'] = git_hash
        if pubsub is not UNSET:
            field_dict['pubsub'] = pubsub

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        async_jobs_running = d.pop("async_jobs_running", UNSET)

        _cache = d.pop("cache", UNSET)
        cache: Union[Unset, CacheMetadata]
        if isinstance(_cache, Unset):
            cache = UNSET
        else:
            cache = CacheMetadata(_cache)

        _environment = d.pop("environment", UNSET)
        environment: Union[Unset, Environment]
        if isinstance(_environment, Unset):
            environment = UNSET
        else:
            environment = Environment(_environment)

        _fs = d.pop("fs", UNSET)
        fs: Union[Unset, FileSystemMetadata]
        if isinstance(_fs, Unset):
            fs = UNSET
        else:
            fs = FileSystemMetadata(_fs)

        git_hash = d.pop("git_hash", UNSET)

        _pubsub = d.pop("pubsub", UNSET)
        pubsub: Union[Unset, Connection]
        if isinstance(_pubsub, Unset):
            pubsub = UNSET
        else:
            pubsub = Connection(_pubsub)

        engine_metadata = cls(
            async_jobs_running=async_jobs_running,
            cache=cache,
            environment=environment,
            fs=fs,
            git_hash=git_hash,
            pubsub=pubsub,
        )

        engine_metadata.additional_properties = d
        return engine_metadata

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SystemInfoCgroupDriverEnum(str, Enum):
    EMPTY = ''
    CGROUPFS = 'cgroupfs'
    SYSTEMD = 'systemd'
    NONE = 'none'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.file_conversion import FileConversion
from ...models.error import Error
from ...models.file_output_format import FileOutputFormat
from ...models.file_source_format import FileSourceFormat
from ...types import Response

def _get_kwargs(
	output_format: FileOutputFormat,
	src_format: FileSourceFormat,
	body: bytes,
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/file/conversion/{src_format}/{output_format}".format(client.base_url, output_format=output_format, src_format=src_format)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
		"content": body,
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, FileConversion, Error]]:
	if response.status_code == 201:
		response_201 = FileConversion.from_dict(response.json())
		return response_201
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, FileConversion, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	output_format: FileOutputFormat,
	src_format: FileSourceFormat,
	body: bytes,
	*,
	client: Client,
) -> Response[Union[Any, FileConversion, Error]]:
	kwargs = _get_kwargs(
		output_format=output_format,
		src_format=src_format,
		body=body,
		client=client,
	)

	response = httpx.post(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	output_format: FileOutputFormat,
	src_format: FileSourceFormat,
	body: bytes,
	*,
	client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
	""" Convert a CAD file from one format to another. If the file being converted is larger than 25MB, it will be performed asynchronously.
If the conversion is performed synchronously, the contents of the converted file (`output`) will be returned as a base64 encoded string.
If the operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint. """

	return sync_detailed(
		output_format=output_format,
		src_format=src_format,
		body=body,
		client=client,
	).parsed


async def asyncio_detailed(
	output_format: FileOutputFormat,
	src_format: FileSourceFormat,
	body: bytes,
	*,
	client: Client,
) -> Response[Union[Any, FileConversion, Error]]:
	kwargs = _get_kwargs(
		output_format=output_format,
		src_format=src_format,
		body=body,
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.post(**kwargs)

	return _build_response(response=response)


async def asyncio(
	output_format: FileOutputFormat,
	src_format: FileSourceFormat,
	body: bytes,
	*,
	client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
	""" Convert a CAD file from one format to another. If the file being converted is larger than 25MB, it will be performed asynchronously.
If the conversion is performed synchronously, the contents of the converted file (`output`) will be returned as a base64 encoded string.
If the operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint. """

	return (
		await asyncio_detailed(
		output_format=output_format,
		src_format=src_format,
		body=body,
			client=client,
		)
	).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.jetstream_config import JetstreamConfig
from ..models.meta_cluster_info import MetaClusterInfo
from ..models.jetstream_stats import JetstreamStats
from ..types import UNSET, Unset

T = TypeVar("T", bound="Jetstream")


@attr.s(auto_attribs=True)
class Jetstream:
    """ """
    config: Union[Unset, JetstreamConfig] = UNSET
    meta: Union[Unset, MetaClusterInfo] = UNSET
    stats: Union[Unset, JetstreamStats] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        config: Union[Unset, str] = UNSET
        if not isinstance(self.config, Unset):
            config = self.config.value
        meta: Union[Unset, str] = UNSET
        if not isinstance(self.meta, Unset):
            meta = self.meta.value
        stats: Union[Unset, str] = UNSET
        if not isinstance(self.stats, Unset):
            stats = self.stats.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if config is not UNSET:
            field_dict['config'] = config
        if meta is not UNSET:
            field_dict['meta'] = meta
        if stats is not UNSET:
            field_dict['stats'] = stats

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _config = d.pop("config", UNSET)
        config: Union[Unset, JetstreamConfig]
        if isinstance(_config, Unset):
            config = UNSET
        else:
            config = JetstreamConfig(_config)

        _meta = d.pop("meta", UNSET)
        meta: Union[Unset, MetaClusterInfo]
        if isinstance(_meta, Unset):
            meta = UNSET
        else:
            meta = MetaClusterInfo(_meta)

        _stats = d.pop("stats", UNSET)
        stats: Union[Unset, JetstreamStats]
        if isinstance(_stats, Unset):
            stats = UNSET
        else:
            stats = JetstreamStats(_stats)

        jetstream = cls(
            config=config,
            meta=meta,
            stats=stats,
        )

        jetstream.additional_properties = d
        return jetstream

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_call_with_price_results_page import ApiCallWithPriceResultsPage
from ...models.error import Error
from ...models.created_at_sort_mode import CreatedAtSortMode
from ...types import Response

def _get_kwargs(
	limit: int,
	page_token: str,
	sort_by: CreatedAtSortMode,
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/user/api-calls?limit={limit}&page_token={page_token}&sort_by={sort_by}".format(client.base_url, limit=limit, page_token=page_token, sort_by=sort_by)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ApiCallWithPriceResultsPage, Error]]:
	if response.status_code == 200:
		response_200 = ApiCallWithPriceResultsPage.from_dict(response.json())
		return response_200
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ApiCallWithPriceResultsPage, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	limit: int,
	page_token: str,
	sort_by: CreatedAtSortMode,
	*,
	client: Client,
) -> Response[Union[Any, ApiCallWithPriceResultsPage, Error]]:
	kwargs = _get_kwargs(
		limit=limit,
		page_token=page_token,
		sort_by=sort_by,
		client=client,
	)

	response = httpx.get(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	limit: int,
	page_token: str,
	sort_by: CreatedAtSortMode,
	*,
	client: Client,
) -> Optional[Union[Any, ApiCallWithPriceResultsPage, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It returns the API calls for the authenticated user.
The API calls are returned in order of creation, with the most recently created API calls first. """

	return sync_detailed(
		limit=limit,
		page_token=page_token,
		sort_by=sort_by,
		client=client,
	).parsed


async def asyncio_detailed(
	limit: int,
	page_token: str,
	sort_by: CreatedAtSortMode,
	*,
	client: Client,
) -> Response[Union[Any, ApiCallWithPriceResultsPage, Error]]:
	kwargs = _get_kwargs(
		limit=limit,
		page_token=page_token,
		sort_by=sort_by,
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.get(**kwargs)

	return _build_response(response=response)


async def asyncio(
	limit: int,
	page_token: str,
	sort_by: CreatedAtSortMode,
	*,
	client: Client,
) -> Optional[Union[Any, ApiCallWithPriceResultsPage, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It returns the API calls for the authenticated user.
The API calls are returned in order of creation, with the most recently created API calls first. """

	return (
		await asyncio_detailed(
		limit=limit,
		page_token=page_token,
		sort_by=sort_by,
			client=client,
		)
	).parsed



from enum import Enum


class AsyncApiCallType(str, Enum):
    FILE_CONVERSION = 'FileConversion'
    FILE_VOLUME = 'FileVolume'
    FILE_MASS = 'FileMass'
    FILE_DENSITY = 'FileDensity'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PluginsInfo")


@attr.s(auto_attribs=True)
class PluginsInfo:
    """ """
    authorization: Union[Unset, List[str]] = UNSET
    log: Union[Unset, List[str]] = UNSET
    network: Union[Unset, List[str]] = UNSET
    volume: Union[Unset, List[str]] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        authorization: Union[Unset, List[str]] = UNSET
        if not isinstance(self.authorization, Unset):
            authorization = self.authorization
        log: Union[Unset, List[str]] = UNSET
        if not isinstance(self.log, Unset):
            log = self.log
        network: Union[Unset, List[str]] = UNSET
        if not isinstance(self.network, Unset):
            network = self.network
        volume: Union[Unset, List[str]] = UNSET
        if not isinstance(self.volume, Unset):
            volume = self.volume

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if authorization is not UNSET:
            field_dict['authorization'] = authorization
        if log is not UNSET:
            field_dict['log'] = log
        if network is not UNSET:
            field_dict['network'] = network
        if volume is not UNSET:
            field_dict['volume'] = volume

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        authorization = cast(List[str], d.pop("authorization", UNSET))

        log = cast(List[str], d.pop("log", UNSET))

        network = cast(List[str], d.pop("network", UNSET))

        volume = cast(List[str], d.pop("volume", UNSET))

        plugins_info = cls(
            authorization=authorization,
            log=log,
            network=network,
            volume=volume,
        )

        plugins_info.additional_properties = d
        return plugins_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ApiCallQueryGroupBy(str, Enum):
    EMAIL = 'email'
    METHOD = 'method'
    ENDPOINT = 'endpoint'
    USER_ID = 'user_id'
    ORIGIN = 'origin'
    IP_ADDRESS = 'ip_address'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="JetstreamApiStats")


@attr.s(auto_attribs=True)
class JetstreamApiStats:
    """ """
    errors: Union[Unset, int] = UNSET
    inflight: Union[Unset, int] = UNSET
    total: Union[Unset, int] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        errors = self.errors
        inflight = self.inflight
        total = self.total

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if errors is not UNSET:
            field_dict['errors'] = errors
        if inflight is not UNSET:
            field_dict['inflight'] = inflight
        if total is not UNSET:
            field_dict['total'] = total

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        errors = d.pop("errors", UNSET)

        inflight = d.pop("inflight", UNSET)

        total = d.pop("total", UNSET)

        jetstream_api_stats = cls(
            errors=errors,
            inflight=inflight,
            total=total,
        )

        jetstream_api_stats.additional_properties = d
        return jetstream_api_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Method(str, Enum):
    OPTIONS = 'OPTIONS'
    GET = 'GET'
    POST = 'POST'
    PUT = 'PUT'
    DELETE = 'DELETE'
    HEAD = 'HEAD'
    TRACE = 'TRACE'
    CONNECT = 'CONNECT'
    PATCH = 'PATCH'
    EXTENSION = 'EXTENSION'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateUser")


@attr.s(auto_attribs=True)
class UpdateUser:
    """ """
    company: Union[Unset, str] = UNSET
    discord: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    github: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        company = self.company
        discord = self.discord
        first_name = self.first_name
        github = self.github
        last_name = self.last_name
        phone = self.phone

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if company is not UNSET:
            field_dict['company'] = company
        if discord is not UNSET:
            field_dict['discord'] = discord
        if first_name is not UNSET:
            field_dict['first_name'] = first_name
        if github is not UNSET:
            field_dict['github'] = github
        if last_name is not UNSET:
            field_dict['last_name'] = last_name
        if phone is not UNSET:
            field_dict['phone'] = phone

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        company = d.pop("company", UNSET)

        discord = d.pop("discord", UNSET)

        first_name = d.pop("first_name", UNSET)

        github = d.pop("github", UNSET)

        last_name = d.pop("last_name", UNSET)

        phone = d.pop("phone", UNSET)

        update_user = cls(
            company=company,
            discord=discord,
            first_name=first_name,
            github=github,
            last_name=last_name,
            phone=phone,
        )

        update_user.additional_properties = d
        return update_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class InvoiceStatus(str, Enum):
    DELETED = 'deleted'
    DRAFT = 'draft'
    OPEN = 'open'
    PAID = 'paid'
    UNCOLLECTIBLE = 'uncollectible'
    VOID = 'void'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import base64
import httpx

from ...client import Client
from ...models import Error
from ...models.file_conversion import FileConversion
from ...types import Response
from ...api.file.get_file_conversion import sync as fc_sync, asyncio as fc_asyncio


def sync(
    id: str,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Get the status of a file conversion. This function automatically base64 decodes the output response if there is one."""

    fc = fc_sync(
        id=id,
        client=client,
    )

    if isinstance(fc, FileConversion) and fc.output != "":
        fc.output = base64.b64decode(fc.output)

    return fc


async def asyncio(
    id: str,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Get the status of a file conversion. This function automatically base64 decodes the output response if there is one."""

    fc = await fc_asyncio(
            id=id,
            client=client,
        )

    if isinstance(fc, FileConversion) and fc.output != "":
        fc.output = base64.b64decode(fc.output)

    return fc



from enum import Enum


class SystemInfoIsolationEnum(str, Enum):
    EMPTY = ''
    DEFAULT = 'default'
    HYPERV = 'hyperv'
    PROCESS = 'process'

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class Environment(str, Enum):
    DEVELOPMENT = 'DEVELOPMENT'
    PREVIEW = 'PREVIEW'
    PRODUCTION = 'PRODUCTION'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import base64
import httpx

from ...client import Client
from ...models import Error
from ...models import FileConversion
from ...models import FileSourceFormat
from ...models import FileOutputFormat
from ...types import Response
from ...api.file.create_file_conversion import sync as fc_sync, asyncio as fc_asyncio

def sync(
    src_format: FileSourceFormat,
    output_format: FileOutputFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Convert a CAD file from one format to another. If the file being converted is larger than a certain size it will be performed asynchronously. This function automatically base64 encodes the request body and base64 decodes the request output."""

    encoded = base64.b64encode(body)

    fc = fc_sync(
        src_format=src_format,
        output_format=output_format,
        body=encoded,
        client=client,
    )

    if isinstance(fc, FileConversion) and fc.output != "":
        fc.output = base64.b64decode(fc.output)

    return fc


async def asyncio(
    src_format: FileSourceFormat,
    output_format: FileOutputFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Convert a CAD file from one format to another. If the file being converted is larger than a certain size it will be performed asynchronously. This function automatically base64 encodes the request body and base64 decodes the request output."""

    encoded = base64.b64encode(body)

    fc = await fc_asyncio(
            src_format=src_format,
            output_format=output_format,
            body=encoded,
            client=client,
        )

    if isinstance(fc, FileConversion) and fc.output != "":
        fc.output = base64.b64decode(fc.output)

    return fc



from enum import Enum


class CreatedAtSortMode(str, Enum):
    CREATED_AT_ASCENDING = 'created-at-ascending'
    CREATED_AT_DESCENDING = 'created-at-descending'

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.error import Error
from ...types import Response

def _get_kwargs(
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/apps/github/callback".format(client.base_url)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, Error]]:
	if response.status_code == 204:
		response_204 = None
		return response_204
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	*,
	client: Client,
) -> Response[Union[Any, Error]]:
	kwargs = _get_kwargs(
		client=client,
	)

	response = httpx.get(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	*,
	client: Client,
) -> Optional[Union[Any, Error]]:
	""" This is different than OAuth 2.0 authentication for users. This endpoint grants access for KittyCAD to access user's repos.
The user doesn't need KittyCAD OAuth authorization for this endpoint, this is purely for the GitHub permissions to access repos. """

	return sync_detailed(
		client=client,
	).parsed


async def asyncio_detailed(
	*,
	client: Client,
) -> Response[Union[Any, Error]]:
	kwargs = _get_kwargs(
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.get(**kwargs)

	return _build_response(response=response)


async def asyncio(
	*,
	client: Client,
) -> Optional[Union[Any, Error]]:
	""" This is different than OAuth 2.0 authentication for users. This endpoint grants access for KittyCAD to access user's repos.
The user doesn't need KittyCAD OAuth authorization for this endpoint, this is purely for the GitHub permissions to access repos. """

	return (
		await asyncio_detailed(
			client=client,
		)
	).parsed



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_token import ApiToken
from ...models.error import Error
from ...types import Response

def _get_kwargs(
	token: str,
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/user/api-tokens/{token}".format(client.base_url, token=token)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ApiToken, Error]]:
	if response.status_code == 200:
		response_200 = ApiToken.from_dict(response.json())
		return response_200
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ApiToken, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	token: str,
	*,
	client: Client,
) -> Response[Union[Any, ApiToken, Error]]:
	kwargs = _get_kwargs(
		token=token,
		client=client,
	)

	response = httpx.get(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	token: str,
	*,
	client: Client,
) -> Optional[Union[Any, ApiToken, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It returns details of the requested API token for the user. """

	return sync_detailed(
		token=token,
		client=client,
	).parsed


async def asyncio_detailed(
	token: str,
	*,
	client: Client,
) -> Response[Union[Any, ApiToken, Error]]:
	kwargs = _get_kwargs(
		token=token,
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.get(**kwargs)

	return _build_response(response=response)


async def asyncio(
	token: str,
	*,
	client: Client,
) -> Optional[Union[Any, ApiToken, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It returns details of the requested API token for the user. """

	return (
		await asyncio_detailed(
		token=token,
			client=client,
		)
	).parsed



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.error import Error
from ...types import Response

def _get_kwargs(
	token: str,
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/user/api-tokens/{token}".format(client.base_url, token=token)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, Error]]:
	if response.status_code == 204:
		response_204 = None
		return response_204
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	token: str,
	*,
	client: Client,
) -> Response[Union[Any, Error]]:
	kwargs = _get_kwargs(
		token=token,
		client=client,
	)

	response = httpx.delete(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	token: str,
	*,
	client: Client,
) -> Optional[Union[Any, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It deletes the requested API token for the user.
This endpoint does not actually delete the API token from the database. It merely marks the token as invalid. We still want to keep the token in the database for historical purposes. """

	return sync_detailed(
		token=token,
		client=client,
	).parsed


async def asyncio_detailed(
	token: str,
	*,
	client: Client,
) -> Response[Union[Any, Error]]:
	kwargs = _get_kwargs(
		token=token,
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.delete(**kwargs)

	return _build_response(response=response)


async def asyncio(
	token: str,
	*,
	client: Client,
) -> Optional[Union[Any, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It deletes the requested API token for the user.
This endpoint does not actually delete the API token from the database. It merely marks the token as invalid. We still want to keep the token in the database for historical purposes. """

	return (
		await asyncio_detailed(
		token=token,
			client=client,
		)
	).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.jetstream_api_stats import JetstreamApiStats
from ..types import UNSET, Unset

T = TypeVar("T", bound="JetstreamStats")


@attr.s(auto_attribs=True)
class JetstreamStats:
    """ """
    accounts: Union[Unset, int] = UNSET
    api: Union[Unset, JetstreamApiStats] = UNSET
    ha_assets: Union[Unset, int] = UNSET
    memory: Union[Unset, int] = UNSET
    reserved_memory: Union[Unset, int] = UNSET
    reserved_store: Union[Unset, int] = UNSET
    store: Union[Unset, int] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        accounts = self.accounts
        api: Union[Unset, str] = UNSET
        if not isinstance(self.api, Unset):
            api = self.api.value
        ha_assets = self.ha_assets
        memory = self.memory
        reserved_memory = self.reserved_memory
        reserved_store = self.reserved_store
        store = self.store

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if accounts is not UNSET:
            field_dict['accounts'] = accounts
        if api is not UNSET:
            field_dict['api'] = api
        if ha_assets is not UNSET:
            field_dict['ha_assets'] = ha_assets
        if memory is not UNSET:
            field_dict['memory'] = memory
        if reserved_memory is not UNSET:
            field_dict['reserved_memory'] = reserved_memory
        if reserved_store is not UNSET:
            field_dict['reserved_store'] = reserved_store
        if store is not UNSET:
            field_dict['store'] = store

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        accounts = d.pop("accounts", UNSET)

        _api = d.pop("api", UNSET)
        api: Union[Unset, JetstreamApiStats]
        if isinstance(_api, Unset):
            api = UNSET
        else:
            api = JetstreamApiStats(_api)

        ha_assets = d.pop("ha_assets", UNSET)

        memory = d.pop("memory", UNSET)

        reserved_memory = d.pop("reserved_memory", UNSET)

        reserved_store = d.pop("reserved_store", UNSET)

        store = d.pop("store", UNSET)

        jetstream_stats = cls(
            accounts=accounts,
            api=api,
            ha_assets=ha_assets,
            memory=memory,
            reserved_memory=reserved_memory,
            reserved_store=reserved_store,
            store=store,
        )

        jetstream_stats.additional_properties = d
        return jetstream_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UnitMetricFormat(str, Enum):
    ATTO = 'atto'
    FEMTO = 'femto'
    PICO = 'pico'
    NANO = 'nano'
    MICRO = 'micro'
    MILLI = 'milli'
    CENTI = 'centi'
    DECI = 'deci'
    METRIC_UNIT = 'metric_unit'
    DECA = 'deca'
    HECTO = 'hecto'
    KILO = 'kilo'
    MEGA = 'mega'
    GIGA = 'giga'
    TERA = 'tera'
    PETA = 'peta'
    EXA = 'exa'

    def __str__(self) -> str:
        return str(self.value)



""" Contains methods for accessing the users API paths: A user is someone who uses the KittyCAD API. Here, we can create, delete, and list users. We can also get information about a user. Operations will only be authorized if the user is requesting information about themselves. """



import os
import pytest
import asyncio

from .client import ClientFromEnv
from .models import FileConversion, FileOutputFormat, FileSourceFormat, User, Pong, ApiCallStatus, FileMass, FileVolume
from .api.file import create_file_conversion_with_base64_helper, create_file_mass, create_file_volume
from .api.meta import ping
from .api.users import get_user_self


def test_get_session():
    # Create our client.
    client = ClientFromEnv()

    # Get the session.
    session: User = get_user_self.sync(client=client)

    assert session is not None

    print(f"Session: {session}")


@pytest.mark.asyncio
async def test_get_session_async():
    # Create our client.
    client = ClientFromEnv()

    # Get the session.
    session: User = await get_user_self.asyncio(client=client)

    assert session is not None

    print(f"Session: {session}")


def test_ping():
    # Create our client.
    client = ClientFromEnv()

    # Get the message.
    message: Pong = ping.sync(client=client)

    assert message is not None

    print(f"Message: {message}")


@pytest.mark.asyncio
async def test_ping_async():
    # Create our client.
    client = ClientFromEnv()

    # Get the message.
    message: Pong = await ping.asyncio(client=client)

    assert message is not None

    print(f"Message: {message}")


def test_file_convert_stl():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.stl"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    fc: FileConversion = create_file_conversion_with_base64_helper.sync(
        client=client,
        body=content,
        src_format=FileSourceFormat.STL,
        output_format=FileOutputFormat.OBJ)

    assert fc is not None

    print(f"FileConversion: {fc}")

    assert fc.id is not None

    assert fc.status == ApiCallStatus.COMPLETED

    print(f"FileConversion: {fc}")


@pytest.mark.asyncio
async def test_file_convert_stl_async():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.stl"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    fc: FileConversion = await create_file_conversion_with_base64_helper.asyncio(client=client, body=content, src_format=FileSourceFormat.STL, output_format=FileOutputFormat.OBJ)

    assert fc is not None

    print(f"FileConversion: {fc}")

    assert fc.id is not None

    print(f"FileConversion: {fc}")


def test_file_mass():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    fm: FileMass = create_file_mass.sync(
        client=client,
        body=content,
        src_format=FileSourceFormat.OBJ,
        material_density=1.0)

    assert fm is not None

    print(f"FileMass: {fm}")

    assert fm.id is not None
    assert fm.mass is not None

    assert fm.to_dict() is not None

    assert fm.status == ApiCallStatus.COMPLETED


def test_file_volume():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    fv: FileVolume = create_file_volume.sync(
        client=client,
        body=content,
        src_format=FileSourceFormat.OBJ)

    assert fv is not None

    print(f"FileVolume: {fv}")

    assert fv.id is not None
    assert fv.volume is not None

    assert fv.to_dict() is not None

    assert fv.status == ApiCallStatus.COMPLETED



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CodeOutput")


@attr.s(auto_attribs=True)
class CodeOutput:
    """ """
    from ..models.output_file import OutputFile
    output_files: Union[Unset, List[OutputFile]] = UNSET
    stderr: Union[Unset, str] = UNSET
    stdout: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.output_file import OutputFile
        output_files: Union[Unset, List[OutputFile]] = UNSET
        if not isinstance(self.output_files, Unset):
            output_files = self.output_files
        stderr = self.stderr
        stdout = self.stdout

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if output_files is not UNSET:
            field_dict['output_files'] = output_files
        if stderr is not UNSET:
            field_dict['stderr'] = stderr
        if stdout is not UNSET:
            field_dict['stdout'] = stdout

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        from ..models.output_file import OutputFile
        output_files = cast(List[OutputFile], d.pop("output_files", UNSET))

        stderr = d.pop("stderr", UNSET)

        stdout = d.pop("stdout", UNSET)

        code_output = cls(
            output_files=output_files,
            stderr=stderr,
            stdout=stdout,
        )

        code_output.additional_properties = d
        return code_output

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.error import Error
from ...types import Response

def _get_kwargs(
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/user".format(client.base_url)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, Error]]:
	if response.status_code == 204:
		response_204 = None
		return response_204
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	*,
	client: Client,
) -> Response[Union[Any, Error]]:
	kwargs = _get_kwargs(
		client=client,
	)

	response = httpx.delete(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	*,
	client: Client,
) -> Optional[Union[Any, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It deletes the authenticated user from KittyCAD's database.
This call will only succeed if all invoices associated with the user have been paid in full and there is no outstanding balance. """

	return sync_detailed(
		client=client,
	).parsed


async def asyncio_detailed(
	*,
	client: Client,
) -> Response[Union[Any, Error]]:
	kwargs = _get_kwargs(
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.delete(**kwargs)

	return _build_response(response=response)


async def asyncio(
	*,
	client: Client,
) -> Optional[Union[Any, Error]]:
	""" This endpoint requires authentication by any KittyCAD user. It deletes the authenticated user from KittyCAD's database.
This call will only succeed if all invoices associated with the user have been paid in full and there is no outstanding balance. """

	return (
		await asyncio_detailed(
			client=client,
		)
	).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.index_info import IndexInfo
from ..types import UNSET, Unset

T = TypeVar("T", bound="RegistryServiceConfig")


@attr.s(auto_attribs=True)
class RegistryServiceConfig:
    """ """
    allow_nondistributable_artifacts_cid_rs: Union[Unset, List[str]] = UNSET
    allow_nondistributable_artifacts_hostnames: Union[Unset, List[str]] = UNSET
    index_configs: Union[Unset, Any] = UNSET
    insecure_registry_cid_rs: Union[Unset, List[str]] = UNSET
    mirrors: Union[Unset, List[str]] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allow_nondistributable_artifacts_cid_rs: Union[Unset,
                                                       List[str]] = UNSET
        if not isinstance(self.allow_nondistributable_artifacts_cid_rs, Unset):
            allow_nondistributable_artifacts_cid_rs = self.allow_nondistributable_artifacts_cid_rs
        allow_nondistributable_artifacts_hostnames: Union[Unset,
                                                          List[str]] = UNSET
        if not isinstance(
                self.allow_nondistributable_artifacts_hostnames,
                Unset):
            allow_nondistributable_artifacts_hostnames = self.allow_nondistributable_artifacts_hostnames
        index_configs = self.index_configs
        insecure_registry_cid_rs: Union[Unset, List[str]] = UNSET
        if not isinstance(self.insecure_registry_cid_rs, Unset):
            insecure_registry_cid_rs = self.insecure_registry_cid_rs
        mirrors: Union[Unset, List[str]] = UNSET
        if not isinstance(self.mirrors, Unset):
            mirrors = self.mirrors

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_nondistributable_artifacts_cid_rs is not UNSET:
            field_dict['allow_nondistributable_artifacts_cid_rs'] = allow_nondistributable_artifacts_cid_rs
        if allow_nondistributable_artifacts_hostnames is not UNSET:
            field_dict['allow_nondistributable_artifacts_hostnames'] = allow_nondistributable_artifacts_hostnames
        if index_configs is not UNSET:
            field_dict['index_configs'] = index_configs
        if insecure_registry_cid_rs is not UNSET:
            field_dict['insecure_registry_cid_rs'] = insecure_registry_cid_rs
        if mirrors is not UNSET:
            field_dict['mirrors'] = mirrors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_nondistributable_artifacts_cid_rs = cast(
            List[str], d.pop("allow_nondistributable_artifacts_cid_rs", UNSET))

        allow_nondistributable_artifacts_hostnames = cast(
            List[str], d.pop("allow_nondistributable_artifacts_hostnames", UNSET))

        index_configs = d.pop("index_configs", UNSET)
        insecure_registry_cid_rs = cast(
            List[str], d.pop(
                "insecure_registry_cid_rs", UNSET))

        mirrors = cast(List[str], d.pop("mirrors", UNSET))

        registry_service_config = cls(
            allow_nondistributable_artifacts_cid_rs=allow_nondistributable_artifacts_cid_rs,
            allow_nondistributable_artifacts_hostnames=allow_nondistributable_artifacts_hostnames,
            index_configs=index_configs,
            insecure_registry_cid_rs=insecure_registry_cid_rs,
            mirrors=mirrors,
        )

        registry_service_config.additional_properties = d
        return registry_service_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kittycad',
 'kittycad.api',
 'kittycad.api.api-calls',
 'kittycad.api.api-tokens',
 'kittycad.api.apps',
 'kittycad.api.beta',
 'kittycad.api.file',
 'kittycad.api.hidden',
 'kittycad.api.meta',
 'kittycad.api.oauth2',
 'kittycad.api.payments',
 'kittycad.api.sessions',
 'kittycad.api.unit',
 'kittycad.api.users',
 'kittycad.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<22.0.0',
 'httpx>=0.15.4,<0.24.0',
 'python-dateutil>=2.8.0,<3.0.0']

setup_kwargs = {
    'name': 'kittycad',
    'version': '0.2.7',
    'description': 'A client library for accessing KittyCAD',
    'long_description': '# kittycad.py\n\nThe Python API client for KittyCAD.\n\n- [PyPI](https://pypi.org/project/kittycad/)\n- [Python docs](https://python.api.docs.kittycad.io/)\n- [KittyCAD API Docs](https://docs.kittycad.io/?lang=python)\n\n## Generating\n\nYou can trigger a build with the GitHub action to generate the client. This will\nautomatically update the client to the latest version based on the spec hosted\nat [api.kittycad.io](https://api.kittycad.io/).\n\nAlternatively, if you wish to generate the client locally, make sure you have\n[Docker installed](https://docs.docker.com/get-docker/) and run:\n\n```bash\n$ make generate\n```\n\n## Contributing\n\nPlease do not change the code directly since it is generated. PRs that change\nthe code directly will be automatically closed by a bot.\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True)
class User:
    """ """
    company: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    discord: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    email_verified: Union[Unset, datetime.datetime] = UNSET
    first_name: Union[Unset, str] = UNSET
    github: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    image: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        company = self.company
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        discord = self.discord
        email = self.email
        email_verified: Union[Unset, str] = UNSET
        if not isinstance(self.email_verified, Unset):
            email_verified = self.email_verified.isoformat()
        first_name = self.first_name
        github = self.github
        id = self.id
        image = self.image
        last_name = self.last_name
        name = self.name
        phone = self.phone
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if company is not UNSET:
            field_dict['company'] = company
        if created_at is not UNSET:
            field_dict['created_at'] = created_at
        if discord is not UNSET:
            field_dict['discord'] = discord
        if email is not UNSET:
            field_dict['email'] = email
        if email_verified is not UNSET:
            field_dict['email_verified'] = email_verified
        if first_name is not UNSET:
            field_dict['first_name'] = first_name
        if github is not UNSET:
            field_dict['github'] = github
        if id is not UNSET:
            field_dict['id'] = id
        if image is not UNSET:
            field_dict['image'] = image
        if last_name is not UNSET:
            field_dict['last_name'] = last_name
        if name is not UNSET:
            field_dict['name'] = name
        if phone is not UNSET:
            field_dict['phone'] = phone
        if updated_at is not UNSET:
            field_dict['updated_at'] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        company = d.pop("company", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        discord = d.pop("discord", UNSET)

        email = d.pop("email", UNSET)

        _email_verified = d.pop("email_verified", UNSET)
        email_verified: Union[Unset, datetime.datetime]
        if isinstance(_email_verified, Unset):
            email_verified = UNSET
        else:
            email_verified = isoparse(_email_verified)

        first_name = d.pop("first_name", UNSET)

        github = d.pop("github", UNSET)

        id = d.pop("id", UNSET)

        image = d.pop("image", UNSET)

        last_name = d.pop("last_name", UNSET)

        name = d.pop("name", UNSET)

        phone = d.pop("phone", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user = cls(
            company=company,
            created_at=created_at,
            discord=discord,
            email=email,
            email_verified=email_verified,
            first_name=first_name,
            github=github,
            id=id,
            image=image,
            last_name=last_name,
            name=name,
            phone=phone,
            updated_at=updated_at,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.uuid import Uuid
from ..models.method import Method
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiCallWithPrice")


@attr.s(auto_attribs=True)
class ApiCallWithPrice:
    """ """
    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    duration: Union[Unset, int] = UNSET
    email: Union[Unset, str] = UNSET
    endpoint: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    ip_address: Union[Unset, str] = UNSET
    method: Union[Unset, Method] = UNSET
    minutes: Union[Unset, int] = UNSET
    origin: Union[Unset, str] = UNSET
    price: Union[Unset, float] = UNSET
    request_body: Union[Unset, str] = UNSET
    request_query_params: Union[Unset, str] = UNSET
    response_body: Union[Unset, str] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status_code: Union[Unset, int] = UNSET
    stripe_invoice_item_id: Union[Unset, str] = UNSET
    token: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_agent: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        duration = self.duration
        email = self.email
        endpoint = self.endpoint
        id = self.id
        ip_address = self.ip_address
        method: Union[Unset, str] = UNSET
        if not isinstance(self.method, Unset):
            method = self.method.value
        minutes = self.minutes
        origin = self.origin
        price = self.price
        request_body = self.request_body
        request_query_params = self.request_query_params
        response_body = self.response_body
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        status_code = self.status_code
        stripe_invoice_item_id = self.stripe_invoice_item_id
        token = self.token
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_agent = self.user_agent
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict['completed_at'] = completed_at
        if created_at is not UNSET:
            field_dict['created_at'] = created_at
        if duration is not UNSET:
            field_dict['duration'] = duration
        if email is not UNSET:
            field_dict['email'] = email
        if endpoint is not UNSET:
            field_dict['endpoint'] = endpoint
        if id is not UNSET:
            field_dict['id'] = id
        if ip_address is not UNSET:
            field_dict['ip_address'] = ip_address
        if method is not UNSET:
            field_dict['method'] = method
        if minutes is not UNSET:
            field_dict['minutes'] = minutes
        if origin is not UNSET:
            field_dict['origin'] = origin
        if price is not UNSET:
            field_dict['price'] = price
        if request_body is not UNSET:
            field_dict['request_body'] = request_body
        if request_query_params is not UNSET:
            field_dict['request_query_params'] = request_query_params
        if response_body is not UNSET:
            field_dict['response_body'] = response_body
        if started_at is not UNSET:
            field_dict['started_at'] = started_at
        if status_code is not UNSET:
            field_dict['status_code'] = status_code
        if stripe_invoice_item_id is not UNSET:
            field_dict['stripe_invoice_item_id'] = stripe_invoice_item_id
        if token is not UNSET:
            field_dict['token'] = token
        if updated_at is not UNSET:
            field_dict['updated_at'] = updated_at
        if user_agent is not UNSET:
            field_dict['user_agent'] = user_agent
        if user_id is not UNSET:
            field_dict['user_id'] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        duration = d.pop("duration", UNSET)

        email = d.pop("email", UNSET)

        endpoint = d.pop("endpoint", UNSET)

        id = d.pop("id", UNSET)

        ip_address = d.pop("ip_address", UNSET)

        _method = d.pop("method", UNSET)
        method: Union[Unset, Method]
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = Method(_method)

        minutes = d.pop("minutes", UNSET)

        origin = d.pop("origin", UNSET)

        price = d.pop("price", UNSET)

        request_body = d.pop("request_body", UNSET)

        request_query_params = d.pop("request_query_params", UNSET)

        response_body = d.pop("response_body", UNSET)

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        status_code = d.pop("status_code", UNSET)

        stripe_invoice_item_id = d.pop("stripe_invoice_item_id", UNSET)

        token = d.pop("token", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_agent = d.pop("user_agent", UNSET)

        user_id = d.pop("user_id", UNSET)

        api_call_with_price = cls(
            completed_at=completed_at,
            created_at=created_at,
            duration=duration,
            email=email,
            endpoint=endpoint,
            id=id,
            ip_address=ip_address,
            method=method,
            minutes=minutes,
            origin=origin,
            price=price,
            request_body=request_body,
            request_query_params=request_query_params,
            response_body=response_body,
            started_at=started_at,
            status_code=status_code,
            stripe_invoice_item_id=stripe_invoice_item_id,
            token=token,
            updated_at=updated_at,
            user_agent=user_agent,
            user_id=user_id,
        )

        api_call_with_price.additional_properties = d
        return api_call_with_price

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.address import Address
from ..models.currency import Currency
from ..types import UNSET, Unset

T = TypeVar("T", bound="Customer")


@attr.s(auto_attribs=True)
class Customer:
    """ """
    address: Union[Unset, Address] = UNSET
    balance: Union[Unset, float] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    currency: Union[Unset, Currency] = UNSET
    delinquent: Union[Unset, bool] = False
    email: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    metadata: Union[Unset, Any] = UNSET
    name: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address: Union[Unset, str] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.value
        balance = self.balance
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        currency: Union[Unset, str] = UNSET
        if not isinstance(self.currency, Unset):
            currency = self.currency.value
        delinquent = self.delinquent
        email = self.email
        id = self.id
        metadata = self.metadata
        name = self.name
        phone = self.phone

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict['address'] = address
        if balance is not UNSET:
            field_dict['balance'] = balance
        if created_at is not UNSET:
            field_dict['created_at'] = created_at
        if currency is not UNSET:
            field_dict['currency'] = currency
        if delinquent is not UNSET:
            field_dict['delinquent'] = delinquent
        if email is not UNSET:
            field_dict['email'] = email
        if id is not UNSET:
            field_dict['id'] = id
        if metadata is not UNSET:
            field_dict['metadata'] = metadata
        if name is not UNSET:
            field_dict['name'] = name
        if phone is not UNSET:
            field_dict['phone'] = phone

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address(_address)

        balance = d.pop("balance", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _currency = d.pop("currency", UNSET)
        currency: Union[Unset, Currency]
        if isinstance(_currency, Unset):
            currency = UNSET
        else:
            currency = Currency(_currency)

        delinquent = d.pop("delinquent", UNSET)

        email = d.pop("email", UNSET)

        id = d.pop("id", UNSET)

        metadata = d.pop("metadata", UNSET)
        name = d.pop("name", UNSET)

        phone = d.pop("phone", UNSET)

        customer = cls(
            address=address,
            balance=balance,
            created_at=created_at,
            currency=currency,
            delinquent=delinquent,
            email=email,
            id=id,
            metadata=metadata,
            name=name,
            phone=phone,
        )

        customer.additional_properties = d
        return customer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.file_conversion import FileConversion
from ...models.file_mass import FileMass
from ...models.file_volume import FileVolume
from ...models.file_density import FileDensity
from ...models.error import Error
from ...types import Response

def _get_kwargs(
	id: str,
	*,
	client: Client,
) -> Dict[str, Any]:
	url = "{}/async/operations/{id}".format(client.base_url, id=id)

	headers: Dict[str, Any] = client.get_headers()
	cookies: Dict[str, Any] = client.get_cookies()

	return {
		"url": url,
		"headers": headers,
		"cookies": cookies,
		"timeout": client.get_timeout(),
	}


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, FileConversion, FileMass, FileVolume, FileDensity, Error]]:
	if response.status_code == 200:
		data = response.json()
		try:
			if not isinstance(data, dict):
				raise TypeError()
			option = FileConversion.from_dict(data)
			return option
		except:
			pass
		try:
			if not isinstance(data, dict):
				raise TypeError()
			option = FileMass.from_dict(data)
			return option
		except:
			pass
		try:
			if not isinstance(data, dict):
				raise TypeError()
			option = FileVolume.from_dict(data)
			return option
		except:
			pass
		try:
			if not isinstance(data, dict):
				raise TypeError()
			option = FileDensity.from_dict(data)
			return option
		except:
			raise
	if response.status_code == 400:
		response_4XX = Error.from_dict(response.json())
		return response_4XX
	if response.status_code == 500:
		response_5XX = Error.from_dict(response.json())
		return response_5XX
	return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, FileConversion, FileMass, FileVolume, FileDensity, Error]]:
	return Response(
		status_code=response.status_code,
		content=response.content,
		headers=response.headers,
		parsed=_parse_response(response=response),
	)


def sync_detailed(
	id: str,
	*,
	client: Client,
) -> Response[Union[Any, FileConversion, FileMass, FileVolume, FileDensity, Error]]:
	kwargs = _get_kwargs(
		id=id,
		client=client,
	)

	response = httpx.get(
		verify=client.verify_ssl,
		**kwargs,
	)

	return _build_response(response=response)


def sync(
	id: str,
	*,
	client: Client,
) -> Optional[Union[Any, FileConversion, FileMass, FileVolume, FileDensity, Error]]:
	""" Get the status and output of an async operation.
This endpoint requires authentication by any KittyCAD user. It returns details of the requested async operation for the user.
If the user is not authenticated to view the specified async operation, then it is not returned.
Only KittyCAD employees with the proper access can view async operations for other users. """

	return sync_detailed(
		id=id,
		client=client,
	).parsed


async def asyncio_detailed(
	id: str,
	*,
	client: Client,
) -> Response[Union[Any, FileConversion, FileMass, FileVolume, FileDensity, Error]]:
	kwargs = _get_kwargs(
		id=id,
		client=client,
	)

	async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
		response = await _client.get(**kwargs)

	return _build_response(response=response)


async def asyncio(
	id: str,
	*,
	client: Client,
) -> Optional[Union[Any, FileConversion, FileMass, FileVolume, FileDensity, Error]]:
	""" Get the status and output of an async operation.
This endpoint requires authentication by any KittyCAD user. It returns details of the requested async operation for the user.
If the user is not authenticated to view the specified async operation, then it is not returned.
Only KittyCAD employees with the proper access can view async operations for other users. """

	return (
		await asyncio_detailed(
		id=id,
			client=client,
		)
	).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.uuid import Uuid
from ..types import UNSET, Unset

T = TypeVar("T", bound="Address")


@attr.s(auto_attribs=True)
class Address:
    """ """
    city: Union[Unset, str] = UNSET
    country: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    street1: Union[Unset, str] = UNSET
    street2: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET
    zip: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        city = self.city
        country = self.country
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        id = self.id
        state = self.state
        street1 = self.street1
        street2 = self.street2
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id
        zip = self.zip

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if city is not UNSET:
            field_dict['city'] = city
        if country is not UNSET:
            field_dict['country'] = country
        if created_at is not UNSET:
            field_dict['created_at'] = created_at
        if id is not UNSET:
            field_dict['id'] = id
        if state is not UNSET:
            field_dict['state'] = state
        if street1 is not UNSET:
            field_dict['street1'] = street1
        if street2 is not UNSET:
            field_dict['street2'] = street2
        if updated_at is not UNSET:
            field_dict['updated_at'] = updated_at
        if user_id is not UNSET:
            field_dict['user_id'] = user_id
        if zip is not UNSET:
            field_dict['zip'] = zip

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        city = d.pop("city", UNSET)

        country = d.pop("country", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        id = d.pop("id", UNSET)

        state = d.pop("state", UNSET)

        street1 = d.pop("street1", UNSET)

        street2 = d.pop("street2", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        zip = d.pop("zip", UNSET)

        address = cls(
            city=city,
            country=country,
            created_at=created_at,
            id=id,
            state=state,
            street1=street1,
            street2=street2,
            updated_at=updated_at,
            user_id=user_id,
            zip=zip,
        )

        address.additional_properties = d
        return address

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
